@@ -25,7 +25,7 @@ from angorapy.agent.dataio import read_dataset_from_storage
 from angorapy.agent.gather import Gatherer, evaluate, EpsilonGreedyGatherer
 from angorapy.agent.ppo.optim import learn_on_batch
 from angorapy.common import policies, const
-from angorapy.common.const import COLORS, BASE_SAVE_PATH, PRETRAINED_COMPONENTS_PATH, STORAGE_DIR
+from angorapy.common.const import COLORS, BASE_SAVE_PATH, PRETRAINED_COMPONENTS_PATH, STORAGE_DIR, PATH_TO_EXPERIMENTS
 from angorapy.common.const import MIN_STAT_EPS
 from angorapy.common.mpi_optim import MpiAdam
 from angorapy.common.policies import BasePolicyDistribution, CategoricalPolicyDistribution, GaussianPolicyDistribution
@@ -68,14 +68,6 @@ n_optimizers = int(mpi_comm.allreduce(is_optimization_process, op=MPI.SUM))
 
 
 class PPOAgent:
-    """Agent using the Proximal Policy Optimization Algorithm for learning.
-    
-    The default is an implementation using two independent models for the critic and the actor. This is of course more
-    expensive than using shared parameters because we need two forward and backward calculations
-    per batch however this is what is used in the original paper and most implementations. During development this also
-    turned out to be beneficial for performance relative to episodes seen in easy tasks (e.g. CartPole) and crucial
-    to make any significant progress in more difficult environments such as LunarLander.
-    """
     policy: tf.keras.Model
     value: tf.keras.Model
     joint: tf.keras.Model
@@ -102,6 +94,14 @@ class PPOAgent:
                  pretrained_components: list = None):
         """ Initialize the PPOAgent with given hyperparameters. Policy and value network will be freshly initialized.
 
+        Agent using the Proximal Policy Optimization Algorithm for learning.
+
+        The default is an implementation using two independent models for the critic and the actor. This is of course more
+        expensive than using shared parameters because we need two forward and backward calculations
+        per batch however this is what is used in the original paper and most implementations. During development this also
+        turned out to be beneficial for performance relative to episodes seen in easy tasks (e.g. CartPole) and crucial
+        to make any significant progress in more difficult environments such as LunarLander.
+
         Args:
             model_builder: a function creating a policy, value and joint model
             environment (gym.Env): the environment in which the agent will learn 
@@ -219,6 +219,7 @@ class PPOAgent:
         self.model_export_dir = "storage/saved_models/exports/"
         self.agent_id = mpi_comm.bcast(f"{round(time.time())}{random.randint(int(1e5), int(1e6) - 1)}", root=0)
         self.agent_directory = f"{BASE_SAVE_PATH}/{self.agent_id}/"
+        self.experiment_directory = f"{PATH_TO_EXPERIMENTS}/{self.agent_id}/"
         if _make_dirs:
             os.makedirs(self.model_export_dir, exist_ok=True)
             os.makedirs(self.agent_directory, exist_ok=True)
@@ -394,7 +395,12 @@ class PPOAgent:
                                                self.joint.trainable_variables))
             self.optimizer.set_weights(optimizer_weights)
 
-        actor = self._make_actor()
+        actor = self._make_actor(
+            horizon=self.horizon,
+            discount=self.discount,
+            lam=self.lam,
+            subseq_length=self.tbptt_length
+        )
 
         cycle_start = None
         full_drill_start_time = time.time()
@@ -413,8 +419,7 @@ class PPOAgent:
             worker_stats = []
 
             for i in worker_collection_ids:
-                stats = actor.collect(actor_joint, self.env, self.horizon, self.discount, self.lam, self.tbptt_length,
-                                      collector_id=i)
+                stats = actor.collect(actor_joint, self.env, collector_id=i)
                 worker_stats.append(stats)
 
             # merge gatherings from all workers
@@ -581,12 +586,16 @@ class PPOAgent:
         self.underflow_history.append(stats.tbptt_underflow)
         self.current_per_receptor_mean = {s: arr.tolist() for s, arr in stats.per_receptor_mean.items()}
 
-    def _make_actor(self) -> Gatherer:
+    def _make_actor(self, horizon, discount, lam, subseq_length) -> Gatherer:
         # create the Gatherer
         return self.gatherer_class(
             worker_id=MPI.COMM_WORLD.rank,
             exp_id=self.agent_id,
-            distribution=self.distribution
+            distribution=self.distribution,
+            horizon=horizon,
+            discount=discount,
+            lam=lam,
+            subseq_length=subseq_length
         )
 
     def optimize(self, dataset: tf.data.TFRecordDataset, epochs: int, batch_size: Union[int, Tuple[int, int]]) -> None:
@@ -763,9 +772,9 @@ class PPOAgent:
             current_lr = self.lr_schedule
 
         # losses
-        pi_loss = "-" if len(self.policy_loss_history) == 0 else f"{round(self.policy_loss_history[-1], 2):6.2f}"
-        v_loss = "-" if len(self.value_loss_history) == 0 else f"{round(self.value_loss_history[-1], 2):8.2f}"
-        ent = "-" if len(self.entropy_history) == 0 else f"{round(self.entropy_history[-1], 2):6.2f}"
+        pi_loss = "  pi  " if len(self.policy_loss_history) == 0 else f"{round(self.policy_loss_history[-1], 2):6.2f}"
+        v_loss = "  v     " if len(self.value_loss_history) == 0 else f"{round(self.value_loss_history[-1], 2):8.2f}"
+        ent = "  ent " if len(self.entropy_history) == 0 else f"{round(self.entropy_history[-1], 2):6.2f}"
 
         # tbptt underflow
         underflow = f"w: {nc}{self.underflow_history[-1]}{ec}; " if self.underflow_history[-1] is not None else ""
