@@ -2,16 +2,15 @@
 
 import abc
 import math
-import os
-from typing import Union, Tuple
+from typing import Union, Tuple, List
 
 import gym
 import numpy as np
 import tensorflow as tf
 
 from angorapy.agent.core import extract_discrete_action_probabilities
-from angorapy.common.layers import StdevLayer
 from angorapy.common.const import EPSILON
+from angorapy.common.layers import StdevLayer, BetaDistributionSpreadLayer
 from angorapy.utilities.util import env_extract_dims
 
 
@@ -49,7 +48,7 @@ class BasePolicyDistribution(abc.ABC):
         pass
 
     @abc.abstractmethod
-    def act_deterministic(self, *args, **kwargs):
+    def act_deterministic(self, *args, **kwargs) -> Tuple[np.ndarray, np.ndarray]:
         pass
 
     @abc.abstractmethod
@@ -106,7 +105,8 @@ class CategoricalPolicyDistribution(BasePolicyDistribution):
         return True
 
     def act(self, log_probabilities: Union[tf.Tensor, np.ndarray]) -> Tuple[np.ndarray, np.ndarray]:
-        """Sample an step_tuple from a discrete step_tuple distribution predicted by the given policy for a given state."""
+        """Sample a step_tuple from a discrete step_tuple distribution predicted by the given policy for a given state.
+        """
         action = self.sample(log_probabilities)
         return action, tf.squeeze(log_probabilities)[action]
 
@@ -170,10 +170,10 @@ class CategoricalPolicyDistribution(BasePolicyDistribution):
 class MultiCategoricalPolicyDistribution(BasePolicyDistribution):
     """Policy distribution for multi-discrete action spaces."""
 
-    def act_deterministic(self, log_probabilities: Union[tf.Tensor, np.ndarray]) -> np.ndarray:
+    def act_deterministic(self, log_probabilities: Union[tf.Tensor, np.ndarray]) -> Tuple[np.ndarray, np.ndarray]:
         action = tf.argmax(log_probabilities, axis=-1)
 
-        return tf.squeeze(action).numpy()
+        return tf.squeeze(action).numpy(), tf.reduce_max(log_probabilities, axis=-1).numpy()
 
     @property
     def short_name(self):
@@ -191,10 +191,10 @@ class MultiCategoricalPolicyDistribution(BasePolicyDistribution):
         sample_probability = tf.gather_nd(tf.squeeze(log_probabilities),
                                           tf.stack([tf.range(len(action)), action], axis=-1))
 
-        return action.astype(np.int), tf.math.reduce_sum(sample_probability, axis=-1)
+        return action.astype(int), tf.math.reduce_sum(sample_probability, axis=-1)
 
     def sample(self, log_probabilities: tf.Tensor) -> np.ndarray:
-        """Sample an step_tuple from the distribution."""
+        """Sample an action from the distribution."""
         assert isinstance(log_probabilities, tf.Tensor) or isinstance(log_probabilities, np.ndarray), \
             f"Policy methods (act_discrete) require Tensors or Numpy Arrays as input, " \
             f"not {type(log_probabilities).__name__}."
@@ -278,8 +278,11 @@ class BaseContinuousPolicyDistribution(BasePolicyDistribution, abc.ABC):
 class GaussianPolicyDistribution(BaseContinuousPolicyDistribution):
     """Gaussian Probability Distribution."""
 
-    def act_deterministic(self, *args, **kwargs):
-        raise NotImplementedError("Act deterministic not defined for Gaussian Distribution.")
+    def act_deterministic(self, means, log_stdevs):
+        actions = means
+        probabilities = self.log_probability(actions, means=means, log_stdevs=log_stdevs)
+
+        return tf.reshape(actions, [-1]).numpy(), tf.squeeze(probabilities).numpy()
 
     @property
     def short_name(self):
@@ -321,10 +324,16 @@ class GaussianPolicyDistribution(BaseContinuousPolicyDistribution):
         Output Shapes:
             - all: (B) or (B, S)
         """
+        tf.debugging.assert_all_finite(samples, "samples not all finite")
+        tf.debugging.assert_all_finite(means, "means not all finite")
+        tf.debugging.assert_all_finite(log_stdevs, "log_stdevs not all finite")
+
         log_likelihoods = (- tf.reduce_sum(log_stdevs, axis=-1)
                            - tf.math.log(2 * np.pi)
                            - (0.5 * tf.reduce_sum(tf.square(((samples - means) / tf.exp(log_stdevs))), axis=-1)))
 
+        tf.debugging.assert_all_finite(log_likelihoods, "log_likelihoods not all finite")
+
         return log_likelihoods
 
     @tf.function
@@ -333,7 +342,7 @@ class GaussianPolicyDistribution(BaseContinuousPolicyDistribution):
 
         Input Shape: (B, A) or (B, S, A) for recurrent
 
-        Since the given r.v.'serialization are independent, the subadditivity property of entropy narrows down to an equality
+        Since the given r.v.'s are independent, the sub-additivity property of entropy narrows down to an equality
         of the joint entropy and the sum of marginal entropies.
         """
         entropy = .5 * tf.math.log(2 * math.pi * math.e * tf.pow(stdevs, 2))
@@ -365,7 +374,7 @@ class GaussianPolicyDistribution(BaseContinuousPolicyDistribution):
         return tf.reduce_sum(log_stdevs, axis=-1)
 
     @tf.function
-    def entropy(self, params: tf.Tensor):
+    def entropy(self, params: Tuple[tf.Tensor, tf.Tensor]):
         """Calculate the joint entropy of Gaussian random variables described by their log standard deviations.
 
         Input Shape: (B, A) or (B, S, A) for recurrent
@@ -431,7 +440,7 @@ class BetaPolicyDistribution(BaseContinuousPolicyDistribution):
 
         actions = self._scale_sample_to_action_range(np.reshape(actions, [-1])).numpy()
 
-        return actions
+        return actions, self.log_probability(actions, alphas, betas)
 
     def sample(self, alphas: tf.Tensor, betas: tf.Tensor):
         """Sample from the Beta distribution."""
@@ -526,13 +535,43 @@ class BetaPolicyDistribution(BaseContinuousPolicyDistribution):
         return tf.keras.Model(inputs=inputs, outputs=[alphas, betas], name="beta_action_head")
 
 
+class RBetaPolicyDistribution(BetaPolicyDistribution):
+    """Beta Policy Distribution with input-independent entropy by reparametrization with mean and a+b."""
+
+    @property
+    def short_name(self):
+        return "rbeta"
+
+    def build_action_head(self, n_actions, input_shape, batch_size):
+        """Build a policy head for the beta distribution, for alpha and beta prediction."""
+        n_actions = n_actions[0]
+
+        inputs = tf.keras.Input(batch_shape=(batch_size,) + tuple(input_shape))
+
+        mu = tf.keras.layers.Dense(n_actions, name="mean", activation="sigmoid",
+                                   kernel_initializer=tf.keras.initializers.Orthogonal(1),
+                                   bias_initializer=tf.keras.initializers.Constant(0.0))(inputs)
+
+        v = BetaDistributionSpreadLayer(n_actions, name="spread")(mu)
+
+        # guarantee a, b > 1 by guaranteeing their sum to be 2
+        v = tf.add(v, 2.)
+
+        # derive alpha and beta
+        alphas = tf.keras.layers.Add(name="alphas")([mu, v])
+        betas = tf.keras.layers.Add(name="betas")([(-mu + 1), v])
+
+        return tf.keras.Model(inputs=inputs, outputs=[alphas, betas], name="beta_action_head")
+
+
 _distribution_short_name_map = {
     "gaussian": GaussianPolicyDistribution,
     "discrete": CategoricalPolicyDistribution,
     "categorical": CategoricalPolicyDistribution,
     "multi-categorical": MultiCategoricalPolicyDistribution,
     "multi-discrete": MultiCategoricalPolicyDistribution,
-    "beta": BetaPolicyDistribution
+    "beta": BetaPolicyDistribution,
+    "rbeta": RBetaPolicyDistribution
 }
 
 
