@@ -3,6 +3,9 @@ import os
 import re
 import shutil
 import sys
+from json import JSONDecodeError
+
+from bokeh import embed
 
 sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
 
@@ -19,7 +22,7 @@ from angorapy.agent.ppo_agent import PPOAgent
 from angorapy.common.const import PATH_TO_EXPERIMENTS, BASE_SAVE_PATH
 from angorapy.utilities.monitor.training_plots import plot_memory_usage, plot_execution_times, plot_preprocessor, \
     plot_reward_progress, plot_loss, plot_length_progress, plot_distribution, compare_reward_progress, \
-    grouped_reward_progress
+    grouped_reward_progress, group_preview
 from angorapy.utilities.statistics import ignore_none
 
 from angorapy.monitor import app
@@ -53,8 +56,11 @@ def overview():
                 with open(os.path.join(exp_path, "progress.json"), "r") as f:
                     progress = json.load(f)
 
-                with open(os.path.join(exp_path, "meta.json"), "r") as f:
-                    meta = json.load(f)
+                try:
+                    with open(os.path.join(exp_path, "meta.json"), "r") as f:
+                        meta = json.load(f)
+                except:
+                    print(eid_m)
 
                 model_available = False
                 if os.path.isfile(os.path.join(model_path, "best/weights.index")):
@@ -85,7 +91,9 @@ def overview():
                         "config_name": meta["config"] if "config" in meta else "unknown",
                         "reward": meta["reward_function"] if "reward_function" in meta else "None",
                         "model": architecture + "[" + model + "]",
-                        "model_available": model_available
+                        "model_available": model_available,
+                        "gatherer": meta["hyperparameters"].get("gatherer", "default"),
+                        "policy": meta["hyperparameters"].get("distribution", "default")
                     }
                 })
 
@@ -94,6 +102,169 @@ def overview():
     return flask.render_template("overview.html", exps=experiments, envs_available=envs_available)
 
 
+@app.route("/groups")
+def view_groups():
+    """Build group view page."""
+    exp_dir = PATH_TO_EXPERIMENTS
+    experiment_paths = [os.path.join(exp_dir, p) for p in os.listdir(exp_dir)]
+
+    experiments_by_groups = {}
+    envs_available = set()
+    for exp_path in experiment_paths:
+
+        eid_m = re.match("[0-9]+", str(exp_path.split("/")[-1]))
+        if eid_m:
+            eid = eid_m.group(0)
+            model_path = os.path.join(BASE_SAVE_PATH, eid)
+
+            if os.path.isfile(os.path.join(exp_path, "progress.json")):
+                with open(os.path.join(exp_path, "progress.json"), "r") as f:
+                    progress = json.load(f)
+
+                with open(os.path.join(exp_path, "meta.json"), "r") as f:
+                    try:
+                        meta = json.load(f)
+                    except JSONDecodeError as jse:
+                        continue
+
+                exp_group = meta.get("experiment_group", "n/a")
+
+                if not exp_group in experiments_by_groups.keys():
+                    experiments_by_groups[exp_group] = {}
+
+                model_available = False
+                if os.path.isfile(os.path.join(model_path, "best/weights.index")):
+                    model_available = True
+
+                reward_threshold = None if meta["environment"]["reward_threshold"] == "None" else float(
+                    meta["environment"]["reward_threshold"])
+                iterations = len(progress["rewards"]["mean"])
+                max_performance = ignore_none(max, progress["rewards"]["mean"])
+                is_success = False
+                if iterations >= 0 and reward_threshold is None:
+                    is_success = "maybe"
+                elif max_performance is not None and max_performance > reward_threshold:
+                    is_success = True
+
+                architecture = "Any" if "architecture" not in meta["hyperparameters"] else meta["hyperparameters"]["architecture"]
+                model = "Any" if "model" not in meta["hyperparameters"] else meta["hyperparameters"]["model"]
+
+                experiments_by_groups[exp_group].update({
+                    eid: {
+                        "env": meta["environment"]["name"],
+                        "date": meta["date"],
+                        "host": meta["host"] if "host" in meta else "unknown",
+                        "iterations": iterations,
+                        "max_reward": max_performance if iterations > 0 else "N/A",
+                        "is_success": is_success,
+                        "bookmark": meta["bookmark"] if "bookmark" in meta else False,
+                        "config_name": meta["config"] if "config" in meta else "unknown",
+                        "reward": meta["reward_function"] if "reward_function" in meta else "None",
+                        "model": architecture + "[" + model + "]",
+                        "model_available": model_available
+                    }
+                })
+
+                envs_available.add(meta["environment"]["name"])
+
+    return flask.render_template("groups.html", exps=experiments_by_groups, envs_available=envs_available)
+
+
+@app.route("/group/", methods=("POST", "GET"))
+def show_group():
+    """Analyze the summarized results of a group of experiments."""
+    ids = request.args.getlist('ids')
+    group_name = request.args.get('group')
+    info = {
+        "ids": ids,
+        "group_name": group_name,
+        "plots": {}
+    }
+
+    progress_reports = {}
+    metas = {}
+    environments = []
+
+    for exp_id in ids:
+        path = f"{PATH_TO_EXPERIMENTS}/{exp_id}"
+        with open(os.path.join(path, "progress.json"), "r") as f:
+            progress = json.load(f)
+
+        with open(os.path.join(path, "meta.json"), "r") as f:
+            meta = json.load(f)
+
+        progress_reports.update({f"{exp_id}": progress})
+        metas[str(exp_id)] = meta
+
+        environments.append(meta["environment"]["name"])
+
+    environments = set(environments)
+    environment_reward_threshold = None
+    if len(environments) == 1:
+        environment_reward_threshold = meta["environment"].get("reward_threshold", None)  # take last exps threshold
+
+    info["plots"]["reward_grouped"] = grouped_reward_progress(
+        {id: (group_name, progress["rewards"]) for id, progress in progress_reports.items()},
+        environment_reward_threshold
+    )
+
+    return flask.render_template("group_analysis.html", info=info)
+
+
+@app.route("/make_group_preview", methods=("POST", "GET"))
+def make_group_preview():
+    """Analyze the summarized results of a group of experiments."""
+    group_names = request.json['names']
+    group_names = group_names.split(",")
+
+    exp_dir = PATH_TO_EXPERIMENTS
+    experiment_paths = [os.path.join(exp_dir, p) for p in os.listdir(exp_dir)]
+
+    environments = {}
+    reward_thresholds = {}
+    experiments_by_groups = {}
+    envs_available = set()
+    for exp_path in experiment_paths:
+
+        eid_m = re.match("[0-9]+", str(exp_path.split("/")[-1]))
+        if eid_m:
+            eid = eid_m.group(0)
+            model_path = os.path.join(BASE_SAVE_PATH, eid)
+
+            if os.path.isfile(os.path.join(exp_path, "progress.json")):
+                with open(os.path.join(exp_path, "progress.json"), "r") as f:
+                    progress = json.load(f)
+
+                with open(os.path.join(exp_path, "meta.json"), "r") as f:
+                    try:
+                        meta = json.load(f)
+                    except JSONDecodeError as jserr:
+                        continue
+
+                exp_group = meta.get("experiment_group", "n/a")
+
+                if exp_group not in group_names:
+                    continue
+
+                reward_threshold = None if meta["environment"]["reward_threshold"] == "None" else float(meta["environment"]["reward_threshold"])
+
+                if not exp_group in experiments_by_groups.keys():
+                    experiments_by_groups[exp_group] = {}
+                    reward_thresholds[exp_group] = reward_threshold
+                    environments[exp_group] = meta["environment"]["name"]
+
+                envs_available.add(meta["environment"]["name"])
+
+                experiments_by_groups[exp_group].update({
+                    eid: progress
+                })
+
+    return flask.render_template("update_group_preview.html",
+                                 plot=group_preview(experiments_by_groups,
+                                                    alt_titles=environments,
+                                                    reward_thresholds=reward_thresholds))
+
+
 @app.route("/benchmarks")
 def benchmarks():
     """Write Benchmark page."""
@@ -201,35 +372,35 @@ def show_experiment(exp_id):
         ))
 
         if "used_memory" in stats:
-            plots["mem_usage"] = plot_memory_usage(stats["used_memory"])
+            plots["mem_usage"] = embed.components(plot_memory_usage(stats["used_memory"]))
 
         if "cycle_timings" in stats:
-            plots["timings"] = plot_execution_times(stats["cycle_timings"],
+            plots["timings"] = embed.components(plot_execution_times(stats["cycle_timings"],
                                                     stats.get("optimization_timings"),
-                                                    stats.get("gathering_timings"))
+                                                    stats.get("gathering_timings")))
 
     if "RewardNormalizationTransformer" in progress["preprocessors"]:
-        plots["normalization"]["reward"] = plot_preprocessor(progress["preprocessors"]["RewardNormalizationTransformer"])
+        plots["normalization"]["reward"] = embed.components(plot_preprocessor(progress["preprocessors"]["RewardNormalizationTransformer"]))
 
     if "StateNormalizationTransformer" in progress["preprocessors"]:
-        plots["normalization"]["state"] = plot_preprocessor(progress["preprocessors"]["StateNormalizationTransformer"])
+        plots["normalization"]["state"] = embed.components(plot_preprocessor(progress["preprocessors"]["StateNormalizationTransformer"]))
 
     reward_threshold = None if meta["environment"]["reward_threshold"] == "None" else float(
         meta["environment"]["reward_threshold"])
 
-    plots["reward_distribution"] = plot_distribution(progress["rewards"]["last_cycle"], "Rewards (Last Cycle)", color=0)
-    plots["length_distribution"] = plot_distribution(progress["lengths"]["last_cycle"], "Episode Lengths (Last Cycle)", color=1)
+    plots["reward_distribution"] = embed.components(plot_distribution(progress["rewards"]["last_cycle"], "Rewards (Last Cycle)", color=0))
+    plots["length_distribution"] = embed.components(plot_distribution(progress["lengths"]["last_cycle"], "Episode Lengths (Last Cycle)", color=1))
     cycles_loaded = []
     if "loaded_at" in stats.keys():
         cycles_loaded = stats["loaded_at"]
-    plots["reward_progress"] = plot_reward_progress(progress["rewards"], cycles_loaded, reward_threshold=reward_threshold)
-    plots["length_progress"] = plot_length_progress(progress["lengths"], cycles_loaded)
-    plots["policy_loss"] = plot_loss(progress["ploss"], progress["rewards"]["mean"], "Policy Loss", color_id=0)
-    plots["value_loss"] = plot_loss(progress["vloss"], progress["rewards"]["mean"], "Value Loss", color_id=1)
-    plots["entropies"] = plot_loss(progress["entropies"], progress["rewards"]["mean"], "Entropy", color_id=2)
+    plots["reward_progress"] = embed.components(plot_reward_progress(progress["rewards"], cycles_loaded, reward_threshold=reward_threshold))
+    plots["length_progress"] = embed.components(plot_length_progress(progress["lengths"], cycles_loaded))
+    plots["policy_loss"] = embed.components(plot_loss(progress["ploss"], progress["rewards"]["mean"], "Policy Loss", color_id=0))
+    plots["value_loss"] = embed.components(plot_loss(progress["vloss"], progress["rewards"]["mean"], "Value Loss", color_id=1))
+    plots["entropies"] = embed.components(plot_loss(progress["entropies"], progress["rewards"]["mean"], "Entropy", color_id=2))
 
     if "per_receptor_mean" in stats.keys():
-        plots["per_receptor_mean"] = plot_per_receptor_mean(stats["per_receptor_mean"])
+        plots["per_receptor_mean"] = embed.components(plot_per_receptor_mean(stats["per_receptor_mean"]))
 
     info.update(dict(
         plots=plots
@@ -256,7 +427,10 @@ def show_comparison():
             progress = json.load(f)
 
         with open(os.path.join(path, "meta.json"), "r") as f:
-            meta = json.load(f)
+            try:
+                meta = json.load(f)
+            except:
+                continue
 
         progress_reports.update({f"{exp_id}": progress})
         metas[str(exp_id)] = meta
@@ -346,6 +520,52 @@ def delete_experiment():
     return {"success": True}
 
 
+@app.route('/delete_experiments', methods=['GET', 'POST'])
+def delete_experiments():
+    """Delete multiple experiments of posted ids."""
+    if request.method == "POST":
+        try:
+            ids = request.json['ids']
+            ids = ids.split(",")
+            ids = list(map(int, ids))
+
+            for id in ids:
+                shutil.rmtree(os.path.join(PATH_TO_EXPERIMENTS, str(id)))
+        except Exception:
+            return {"success": False}
+    else:
+        return {"success": False}
+
+    return {"success": True}
+
+
+@app.route('/regroup_experiments', methods=['GET', 'POST'])
+def regroup_experiments():
+    """Assign new group to multiple experiments of posted ids."""
+    if request.method == "POST":
+        try:
+            ids = request.json['ids']
+            ids = ids.split(",")
+            ids = list(map(int, ids))
+            new_group = request.json['group']
+
+            for id in ids:
+                with open(os.path.join(PATH_TO_EXPERIMENTS, str(id), "meta.json"), "r") as f:
+                    data = json.load(f)
+
+                data.update({"experiment_group": new_group})
+
+                with open(os.path.join(PATH_TO_EXPERIMENTS, str(id), "meta.json"), "w") as f:
+                    json.dump(data, f)
+
+        except Exception:
+            return {"success": False}
+    else:
+        return {"success": False}
+
+    return {"success": True}
+
+
 @app.route("/evaluate", methods=("POST", "GET"))
 def evaluate():
     """Evaluate an agent."""
